##############################
# R Group Talk - 31/01/2019  #
##############################

# NA

vec <- c(2, 4, NA, 2)
vec

# Check whether a value is missing:

vec[3] > 9999 # NA is NOT a large number, like in Stata
vec[3] < (-9999) # NA is not a large negative number like in SAS
vec[3] == NA # NA is NOT even a number 
is.na(vec[3]) # Need this function 
is.na(vec) # SImilarly on a matrix, data.frame...
class(NA)

# Different data types

vec.integ<-c(as.integer(1),as.integer(2),NA_integer_) # Missing value is an integer
vec.integ
is.na(vec.integ)
class(vec.integ[3])

vec.real<-c(1.2,2.12, NA_real_) # Missing value is a real number
vec.real
is.na(vec.real)
class(vec.real[3])

vec.complex<-c(as.complex(1),complex(real = 1, imaginary = 2), NA_complex_) # Missing value is a complex number
vec.complex
is.na(vec.complex)
class(vec.complex[3])

vec.complex2<-c(1,complex(real <- 1, imaginary <- 2), NA) # Is this equivalent?
vec.complex2
is.na(vec.complex2)
class(vec.complex2[3])
class(as.complex(NA))

vec.char<-c("a","b",NA_character_) # Missing value is a character
vec.char
is.na(vec.char)
class(vec.char[3])

# Be careful, "NA" not the same as NA! Imagine we define vector of chemical elements
vec.char2 <-c("CL", "NA", "O",NA)
is.na(vec.char2)

vec.err <- c( 2, 3, 4, "NA") #Cannot mix data types in vector
vec.err    # They were all considered character
is.na(vec.err)

# Useful functions

vec1<-c(1,2,3,4,3,4,NA,5,6,3)
vec2<-c(1,5,5,6,3)

anyNA(vec1)
anyNA(vec2)

is.na(vec2[3])
vec2
is.na(vec2)<-3
vec2
is.na(vec2[3])

# Factors

vec<-factor(c(1,2,3,1,2,3,2,3,2,1))
vec[3]<-NA
vec # Defined slightly differently as <NA>

vec<-addNA(vec)
vec #Now considered like a different category

vec<-factor(c("a","b","NA"))
vec[1]<-NA
vec
is.na(vec)


# Applying functions on vectors with NA

vec<-c(2,3,4,2,3,NA,2,3)
sum(vec)
mean(vec)
sum(vec, na.rm = T)
mean(vec, na.rm = T)

# Regression models

vec<-c(2,3,4,5,2,3,4)
vec2<-rnorm(length(vec))
fit<-lm(vec2~vec)
summary(fit) # 5 degrees of freedom

is.na(vec)<-2:3 # Introduce missing data
fit.CR<-lm(vec2~vec)
summary(fit.CR) # 3 degrees of freedom

library(nlme)
fit.gls<-gls(vec2~vec) # This time it fails. Why?

fit.CR.omit<-lm(vec2~vec, na.action = na.omit)
summary(fit.CR.omit) # As per the default

fit.CR.fail<-lm(vec2~vec, na.action = na.fail) # As in gls

fit.CR.pass<-lm(vec2~vec, na.action = na.pass) # tried to use x, but NA

fit.CR.exclude<-lm(vec2~vec, na.action = na.exclude) # Apparently the same as na.omit
summary(fit.CR.exclude)

# Difference between na.omit and na.exclude?

predict(fit.CR.omit)
predict(fit.CR.exclude)

# Can set it as a default option within the R session:

options(na.action = na.fail)
fit.CR<-lm(vec2~vec)
options(na.action = na.omit)

# Weird behavior of NA:

vec<-c(1,4,2,3)
vec

vec[c(2,4)]
vec[c(2,NA)]
vec[c(NA,NA)] # What did just happen?

letters[c(2,4)]
letters[c(2,NA)]
letters[c(NA,NA)]

month.name[c(2,4)]
month.name[c(2,NA)]
month.name[c(NA,NA)]

class(c(2,NA))
class(c(NA,NA))

letters[T]
letters[c(T,F)]
letters[c(T,F,NA)]

letters[c(NA_integer_, NA_integer_)] # Bingo!


# NaN

2/0
-2/0
0/0

a<-0/0
a<2
a>2
a == NaN
is.nan(a)
is.na(a)
b<-NA
is.nan(b)

is.nan(vec)<-1 #Does not exist

is.finite(2/0)
is.infinite(99999999999999999999999999999999999999999999999999999999)

# Other ways to produce nan

sin(Inf)
3/0-2/0
Inf/Inf
log(-2)
sqrt(-2)

# Using functions on vectors with NaN

vec<-c(2,3,4,2,3,NaN)
sum(vec)
sum(vec,na.rm=T)

# Fitting regression model

vec<-c(2,3,4,5,4,3)
vec2<-rnorm(length(vec))
fit<-lm(vec2~vec)
summary(fit)

vec[2]<-NaN
fit<-lm(vec2~vec)
summary(fit)

fit<-lm(vec2~vec, na.action = na.fail)

# NULL

a <- NULL

a < (-2000)
a > 2000
a == NULL
length(a)
b<-NA
length(b)

is.null(a)
class(a)
class(b)

b <- c(2, 1, NULL)
b
is.null(b)
length(b)

d<-c("a", "b", "NULL", NULL)
d

is.null(e) # Still need to define the object first

# Apply functions to null objects
e <- NULL
sum(e)
mean(e)

f<-c(2,3,2,3,4,5)
lm(f~e)

f<-e+1
f
length(f)
class(f)

f<-e+as.integer(1)
f
length(f)
class(f)

e
f
g <- c(e,2,3)
g
class(g)
g <- c(f,2,3)
g
class(g)

letters[c(NULL, NULL)]
month.name[c(NULL,2)]

ncol(e)
length(e)

# Differences between NULL and numeric(0)

vec1 <- NULL
vec2 <- vec1+1
vec1
vec2
identical(vec1,vec2)
is.null(vec2)

attr(vec2, "name")<-"nothing"
vec2
attr(vec1, "name")<-"nothing"

# DIfferences between NA (NaN) and NULL

vec <- c(1,2,3) 
vec[4]   # In vectors element beyond index is NA

lis<-list(a=1, b=2, c=3)
lis$d   # In lists non-existing element is NULL

vec<-c(1,2,NA,NULL)
vec # In vectors only NA remain, NULL disappear
lis<-list(1,2,NA,NULL)
lis # In list NULL remains as an element of the list as well

vec[1]<-NULL # Not possible to set element of a vector to null
lis[1]<-NULL 
lis # NULL in lists deletes element

lis<-list(NULL, lis) # To add a null element
lis

vec1 <- c(NULL, NULL, NULL)
vec2 <- NULL

class(vec1)
vec1
vec1[3]
class(vec2)
vec2
vec2[3] # Just because it is a null vector
vec1[4] # As above

vec3 <- c(NA, NA, NA)
vec4 <- NA

class(vec3)
vec3
vec3[3]
class(vec4)
vec4
vec4[3] # Non-existing elements are NA anyway.
vec3[4] # As above

vec5 <- c(NaN, NaN, NaN)
vec6 <- NaN

class(vec5)
vec5
vec5[3]
class(vec6)
vec6
vec6[3] # It does not exist, so it becomes NA
vec5[4] # As above

# Data frames

#empty data.frame
df1 <- data.frame(vec1=NA,vec2=NA, vec3=NA)
df2 <- data.frame(vec1=NULL, vec2=NULL, vec3=NULL)
df3 <- data.frame(vec1=NaN, vec2=NaN, vec3=NaN)

df1
df2
df3

dim(df1)
dim(df2)
dim(df3)

df4<-NULL
dim(df4)

# Adding rows to dataframe

df1 <- rbind(df1, data.frame(vec1=1, vec2=2,vec3=3))
df1 <- rbind(df1, data.frame(vec1=1, vec2=NA,vec3=3))
df1
dim(df1) # Similarly for NaN

df2 <- rbind(df2, data.frame(vec1=1, vec2=2, vec3=3))
dim(df2)
df2 <- rbind(df2, data.frame(vec1=1, vec2=NULL, vec3=3))
df2 <- rbind(df2, data.frame(vec1=1, vec2=NA, vec3=3))

#Useful functions to handle missing data

View(airquality)
complete.cases(airquality) # useful for matrices and dataframes

vec<-c(2,3,NA)
complete.cases(vec) # Equivalent to is.na

vec2<-c(3,3,3)
lis<-list(vec,vec2)
complete.cases(lis)
lis2<-list(vec2,vec)
complete.cases(lis2) # Careful with lists
vec3<-c(2,2)
complete.cases(vec,vec3) # Only if they have same length

library(mice)
md.pattern(airquality)

# NA, NaN, NULL are reserved words

NA<-3
NaN<-3
NULL<-3

NAN<-3
na<-"NA"
is.na(na)
